#define BUILD_NUMBER \
	"build-15389592"
#define BUILD_NUMBER_NUMERIC \
	15389592
#define BUILD_NUMBER_NUMERIC_STRING \
	"15389592"
#define PRODUCT_BUILD_NUMBER \
	"product-build-17716"
#define PRODUCT_BUILD_NUMBER_NUMERIC \
	17716
#define PRODUCT_BUILD_NUMBER_NUMERIC_STRING \
	"17716"
